package Net::HyperTap;

use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(
	
);

our $VERSION = '0.01';
$VERSION = eval $VERSION;  # see L<perlmodstyle>

require MIME::Base64;
require Linux::TunTap;
require LWP::UserAgent;
require HTTP::Daemon;
require HTTP::Request;
require HTTP::Response;
require HTTP::Status;
require Getopt::Long;

# Configuration
my $address   = '127.0.0.1';
my $port      = 80;
my $interface = 'tap0';
my $timeout   = 0.2;
my $mode      = "";

my $result = Getopt::Long::GetOptions ("host|address|h=s" => \$address,
                                       "port|p=i" => \$port,
                                       "interface|device|i=s" => \$interface,
                                       "timeout|t=i" => \$timeout,
                                       "mode|m=s" => \$mode
                                      );

my $tap = new Linux::TunTap(
                            NAME => $interface
                           );

sub server {
 my $daemon = HTTP::Daemon->new(
                                LocalAddr => $address,
                                LocalPort => $port
                               );

 while (1) {
  my ($connection, $peer) = $daemon->accept();
  print "Accepted HTTP connection from $peer.\n";

  my $request = $connection->get_request();

  my $method = $request->method();
  print "Request method is $method.\n";

  if ($method eq 'POST') {
   my $content = $request->content();
   print ">>> =================\n$content\n===================\n";

   my $packet = MIME::Base64::decode($content);
   $tap->put_raw($packet);
   print "Wrote packet to device.\n";
  }

  my $status = 204;
  my $content = '';

  my $packet = $tap->get_raw($timeout);

  if (defined($packet)) {
   $status = 200;
   $content = MIME::Base64::encode($packet);
   print "<<< =================\n$content\n===================\n";
  }

  print "Sending status $status.\n";

  my $response = HTTP::Response->new($status);
  $response->content($content);

  $connection->send_response($response);
 }
}

sub client {
 while (1) {
  my $packet = $tap->get_raw($timeout);

  my $method = 'GET';
  my $content = '';

  print "Request method $method.\n";

  if (defined($packet)) {
   $method = 'POST';
   $content = MIME::Base64::encode($packet);

   print "<<< =================\n$content\n===================\n";
  }

  my $request = HTTP::Request->new($method => "http://$address/");
  $request->content_type('text/plain');
  $request->content($content);

  my $agent = LWP::UserAgent->new();
  my $response = $agent->request($request);

  if ($response->is_success()) {
   my $code = $response->code();
   print "Got status code $code.\n";

   if ($code == 200) {
    my $content = $response->content();
    print ">>> =================\n$content\n===================\n";

    my $packet = MIME::Base64::decode($content);
    $tap->put_raw($packet);
    print "Wrote packet to device.\n";
   }
  }
 }
}

1;
__END__

=head1 NAME

Net::HyperTap - Module for the HyperTap HTTP tunnel software

=head1 SYNOPSIS

  use Net::HyperTap;

=head1 DESCRIPTION

Net::HyperTap encapsulates the functionality of the HyperTap HTTP tunnel softare. It can be used
to route arbitrary IP datagrams or even Ethernet frames through a HTTP connection (yes, this includes
restrictive Proxy servers ...).

=head2 EXPORT

None by default.


=head1 SEE ALSO

=head1 AUTHOR

Dominik George, E<lt>dev@naturalnik.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2010 by Dominik George

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.1 or,
at your option, any later version of Perl 5 you may have available.


=cut
